@@ -100,6 +100,7 @@
  *	automated updates after select() (i always missed that with OSS!).
  */
 
+#define _GNU_SOURCE
 #include <stdio.h>
 #include <unistd.h>
 #include <fcntl.h>
@@ -146,7 +147,8 @@
 
 #define MIXER_MIN_X	(18)			/* abs minimum: 18 */
 #define	MIXER_TEXT_Y	(10)
-#define	MIXER_MIN_Y	(MIXER_TEXT_Y + 3)	/* abs minimum: 11 */
+#define MIXER_CBAR_STD_HGT (10)
+#define	MIXER_MIN_Y	(MIXER_TEXT_Y + 6)	/* abs minimum: 16 */
 
 #define MIXER_BLACK	(COLOR_BLACK)
 #define MIXER_DARK_RED  (COLOR_RED)
@@ -167,6 +169,8 @@
 /* --- views --- */
 enum {
   VIEW_CHANNELS,
+  VIEW_PLAYBACK,
+  VIEW_CAPTURE,
   VIEW_HELP,
   VIEW_PROCINFO
 };
@@ -177,17 +181,21 @@ static WINDOW	*mixer_window = NULL;
 static int	 mixer_needs_resize = 0;
 static int	 mixer_minimize = 0;
 static int	 mixer_no_lrcorner = 0;
-static int	 mixer_view = VIEW_CHANNELS;
+static int	 mixer_view = VIEW_PLAYBACK;
+static int	 mixer_view_saved = VIEW_PLAYBACK;
 static int	 mixer_max_x = 0;
 static int	 mixer_max_y = 0;
 static int	 mixer_ofs_x = 0;
 static float	 mixer_extra_space = 0;
 static int	 mixer_cbar_height = 0;
+static int       mixer_text_y = MIXER_TEXT_Y;
 
 static char	 card_id[64] = "default";
 static snd_mixer_t *mixer_handle;
 static char	 mixer_card_name[128];
 static char	 mixer_device_name[128];
+static int	 mixer_level = 0;
+static struct snd_mixer_selem_regopt mixer_options;
 
 /* mixer bar channel : left or right */
 #define MIXER_CHN_LEFT		0
@@ -227,6 +235,7 @@ static int       mixer_changed_state = 1;
 
 /* split scontrols */
 static int	 mixer_n_elems = 0;
+static int	 mixer_n_view_elems = 0;
 static int	 mixer_n_vis_elems = 0;
 static int	 mixer_first_vis_elem = 0;
 static int	 mixer_focus_elem = 0;
@@ -235,6 +244,7 @@ static int *mixer_grpidx;
 static int *mixer_type;
 
 static int	 mixer_volume_delta[2];		/* left/right volume delta in % */
+static int	 mixer_volume_absolute = -1;	/* absolute volume settings in % */
 static int	 mixer_balance_volumes = 0;	/* boolean */
 static unsigned	 mixer_toggle_mute = 0;		/* left/right mask */
 static unsigned	 mixer_toggle_capture = 0;	/* left/right mask */
@@ -250,15 +260,17 @@ static int	 mixer_help_xoffs = 0;
 static int	 mixer_help_yoffs = 0;
 static char     *mixer_help_text =
 (
- "\n"
  " Esc     exit alsamixer\n"
- " F1      show Help screen\n"
- " F2      show /proc info screen\n"
+ " F1 ?    show Help screen\n"
+ " F2 /    show /proc info screen\n"
+ " F3      show Playback controls only\n"
+ " F4      show Capture controls only\n"
+ " F5      show all controls\n"
+ " Tab     toggle view mode\n"
  " Return  return to main screen\n"
  " Space   toggle Capture facility\n"
- " Tab     toggle ExactMode\n"
- " m M     mute both channels\n"
- " < >     mute left/right channel\n"
+ " m M     toggle mute on both channels\n"
+ " < >     toggle mute on left/right channel\n"
  " Up      increase left and right volume\n"
  " Down    decrease left and right volume\n"
  " Right   move (scroll) to the right next channel\n"
@@ -275,6 +287,7 @@ enum {
   DC_BACK,
   DC_TEXT,
   DC_PROMPT,
+  DC_CBAR_FRAME,
   DC_CBAR_MUTE,
   DC_CBAR_NOMUTE,
   DC_CBAR_CAPTURE,
@@ -328,8 +341,9 @@ mixer_init_draw_contexts (void)
   mixer_init_dc ('.', DC_BACK, MIXER_WHITE, MIXER_BLACK, A_NORMAL);
   mixer_init_dc ('.', DC_TEXT, MIXER_YELLOW, MIXER_BLACK, A_BOLD);
   mixer_init_dc ('.', DC_PROMPT, MIXER_DARK_CYAN, MIXER_BLACK, A_NORMAL);
-  mixer_init_dc ('M', DC_CBAR_MUTE, MIXER_CYAN, MIXER_BLACK, A_BOLD);
-  mixer_init_dc (ACS_HLINE, DC_CBAR_NOMUTE, MIXER_CYAN, MIXER_BLACK, A_BOLD);
+  mixer_init_dc ('.', DC_CBAR_FRAME, MIXER_CYAN, MIXER_BLACK, A_BOLD);
+  mixer_init_dc ('M', DC_CBAR_MUTE, MIXER_DARK_CYAN, MIXER_BLACK, A_NORMAL);
+  mixer_init_dc ('O', DC_CBAR_NOMUTE, MIXER_WHITE, MIXER_GREEN, A_BOLD);
   mixer_init_dc ('x', DC_CBAR_CAPTURE, MIXER_DARK_RED, MIXER_BLACK, A_BOLD);
   mixer_init_dc ('-', DC_CBAR_NOCAPTURE, MIXER_GRAY, MIXER_BLACK, A_NORMAL);
   mixer_init_dc (' ', DC_CBAR_EMPTY, MIXER_GRAY, MIXER_BLACK, A_DIM);
@@ -342,7 +356,6 @@ mixer_init_draw_contexts (void)
   mixer_init_dc ('.', DC_ANY_4, MIXER_WHITE, MIXER_BLUE, A_BOLD);
 }
 
-#define	DC_CBAR_FRAME	(DC_CBAR_MUTE)
 #define	DC_FRAME	(DC_PROMPT)
 
 
@@ -452,12 +465,12 @@ mixer_cbar_get_pos (int  elem_index,
   x = mixer_ofs_x;
   x += (3 + 2 + 3 + 1) * elem_index + mixer_extra_space * (elem_index + 1);
 
-  if (MIXER_TEXT_Y + 10 < mixer_max_y)
-    y = mixer_max_y / 2 + 3;
+  if (mixer_text_y + MIXER_CBAR_STD_HGT < mixer_max_y)
+    y = (mixer_text_y + mixer_cbar_height) / 2 - 1 + mixer_max_y / 2;
   else
-    y = (mixer_max_y + 1) / 2 + 3;
-  y += mixer_cbar_height / 2;
-  
+    y = mixer_text_y - 1 + mixer_cbar_height;
+  if (y >= mixer_max_y - 1)
+    y = mixer_max_y - 2;
   if (x_p)
     *x_p = x;
   if (y_p)
@@ -510,6 +523,18 @@ mixer_calc_volume(snd_mixer_elem_t *elem,
   return CLAMP(vol1, min, max);
 }
 
+static int
+mixer_convert_volume(snd_mixer_elem_t *elem,
+		  int vol, int type)
+{
+  long min, max;
+  if (type != MIXER_ELEM_CAPTURE)
+    snd_mixer_selem_get_playback_volume_range(elem, &min, &max);
+  else
+    snd_mixer_selem_get_capture_volume_range(elem, &min, &max);
+  return  mixer_conv(vol, 0, 100, min, max);
+}
+
 /* update enum list */
 static void update_enum_list(snd_mixer_elem_t *elem, int chn, int delta)
 {
@@ -545,7 +570,7 @@ mixer_write_cbar (int elem_index)
 
   if (mixer_sid == NULL)
     return;
-  
+
   sid = (snd_mixer_selem_id_t *)(((char *)mixer_sid) + snd_mixer_selem_id_sizeof() * mixer_grpidx[elem_index]);
   elem = snd_mixer_find_selem(mixer_handle, sid);
   if (elem == NULL)
@@ -567,6 +592,7 @@ mixer_write_cbar (int elem_index)
    */
   if ((mixer_volume_delta[MIXER_CHN_LEFT] ||
        mixer_volume_delta[MIXER_CHN_RIGHT] ||
+       mixer_volume_absolute != -1 ||
        mixer_balance_volumes) &&
       (mixer_type[elem_index] & MIXER_ELEM_HAS_VOLUME)) {
     int mono;
@@ -577,16 +603,20 @@ mixer_write_cbar (int elem_index)
     else
 	joined = snd_mixer_selem_has_capture_volume_joined(elem);
     mono |= joined;
-    if (mono && !mixer_volume_delta[MIXER_CHN_LEFT])
-      mixer_volume_delta[MIXER_CHN_LEFT] = mixer_volume_delta[MIXER_CHN_RIGHT];
-    vleft = mixer_calc_volume(elem, mixer_volume_delta[MIXER_CHN_LEFT], type, chn_left);
-    vbalance = vleft;
-    if (! mono) {
-      vright = mixer_calc_volume(elem, mixer_volume_delta[MIXER_CHN_RIGHT], type, chn_right);
-      vbalance += vright;
-      vbalance /= 2;
+    if (mixer_volume_absolute != -1) {
+      vbalance = vright = vleft = mixer_convert_volume(elem, mixer_volume_absolute, type);
     } else {
-      vright = vleft;
+      if (mono && !mixer_volume_delta[MIXER_CHN_LEFT])
+        mixer_volume_delta[MIXER_CHN_LEFT] = mixer_volume_delta[MIXER_CHN_RIGHT];
+      vleft = mixer_calc_volume(elem, mixer_volume_delta[MIXER_CHN_LEFT], type, chn_left);
+      vbalance = vleft;
+      if (! mono) {
+        vright = mixer_calc_volume(elem, mixer_volume_delta[MIXER_CHN_RIGHT], type, chn_right);
+        vbalance += vright;
+        vbalance /= 2;
+      } else {
+        vright = vleft;
+      }
     }
 
       if (joined) {
@@ -627,7 +657,7 @@ mixer_write_cbar (int elem_index)
   /* mute
    */
   if (mixer_type[elem_index] & MIXER_ELEM_MUTE_SWITCH) {
-    if (mixer_toggle_mute && snd_mixer_selem_has_playback_switch(elem)) {
+    if (mixer_toggle_mute) {
       if (snd_mixer_selem_has_playback_switch_joined(elem)) {
 	snd_mixer_selem_get_playback_switch(elem, chn_left, &sw);
 	snd_mixer_selem_set_playback_switch_all(elem, !sw);
@@ -680,18 +710,93 @@ mixer_write_cbar (int elem_index)
   }
 
   mixer_volume_delta[MIXER_CHN_LEFT] = mixer_volume_delta[MIXER_CHN_RIGHT] = 0;
+  mixer_volume_absolute = -1;
   mixer_balance_volumes = 0;
 }
 
 
+static void draw_blank(int x, int y, int lines)
+{
+  int i;
+
+  mixer_dc (DC_TEXT);
+  for (i = 0; i < lines; i++)
+    mvaddstr (y - i, x, "         ");
+}
+
+/* show the current view mode */
+static void display_view_info(void)
+{
+  mixer_dc (DC_PROMPT);
+  mvaddstr (3, 2, "View:  Playback  Capture  All ");
+  mixer_dc (DC_TEXT);
+  switch (mixer_view) {
+  case VIEW_PLAYBACK:
+    mvaddstr (3, 8, "[Playback]");
+    break;
+  case VIEW_CAPTURE:
+    mvaddstr (3, 18, "[Capture]");
+    break;
+  default:
+    mvaddstr (3, 27, "[All]");
+    break;
+  }
+}
+
+/* show the information of the focused item */
+static void display_item_info(int elem_index, snd_mixer_selem_id_t *sid, char *extra_info)
+{
+  char string[64], idxstr[10];
+  int idx;
+  int i, xlen = mixer_max_x - 8;
+  if (xlen > sizeof(string) - 1)
+    xlen = sizeof(string) - 1;
+  mixer_dc (DC_PROMPT);
+  mvaddstr (4, 2, "Item: ");
+  mixer_dc (DC_TEXT);
+  idx = snd_mixer_selem_id_get_index(sid);
+  if (idx > 0)
+    snprintf(idxstr, sizeof(idxstr), " %i", snd_mixer_selem_id_get_index(sid));
+  snprintf(string, sizeof(string), "%s%s%s%s",
+	   snd_mixer_selem_id_get_name(sid),
+	   (mixer_type[elem_index] & MIXER_ELEM_CAPTURE_SUFFIX) ? " Capture" : "",
+	   idx > 0 ? idxstr : "",
+	   extra_info);
+  for (i = strlen(string); i < sizeof(string) - 1; i++)
+    string[i] = ' ';
+  string[xlen] = '\0';
+  addstr(string);
+}
+
+/* show the bar item name */
+static void display_item_name(int x, int y, int elem_index, snd_mixer_selem_id_t *sid)
+{
+  const char *suffix;
+  char string1[9], string[9];
+  int i;
+
+  mixer_dc (elem_index == mixer_focus_elem ? DC_CBAR_FOCUS_LABEL : DC_CBAR_LABEL);
+  if (mixer_type[elem_index] & MIXER_ELEM_CAPTURE_SUFFIX)
+    suffix = " Capture";
+  else
+    suffix = "";
+  if (snd_mixer_selem_id_get_index(sid) > 0)
+    snprintf(string1, sizeof(string1), "%s%s %d", snd_mixer_selem_id_get_name(sid),
+	     suffix, snd_mixer_selem_id_get_index(sid));
+  else
+    snprintf(string1, sizeof(string1), "%s%s", snd_mixer_selem_id_get_name(sid), suffix);
+  string[8] = 0;
+  for (i = 0; i < 8; i++)
+    string[i] = ' ';
+  memcpy(string + (8 - strlen (string1)) / 2, string1, strlen(string1));
+  mvaddstr (y, x, string);
+}
+
 static void display_enum_list(snd_mixer_elem_t *elem, int y, int x)
 {
-  int i, cury, ch, err;
+  int cury, ch, err;
 
-  /* clear */
-  mixer_dc(DC_TEXT);
-  for (i = mixer_cbar_height + 3, cury = y; i > 0; i--, cury--)
-    mvaddstr(cury, x, "       ");
+  draw_blank(x, y, mixer_cbar_height + (mixer_view == VIEW_PLAYBACK ? 5 : 6));
   
   cury = y - 4;
   for (ch = 0; ch < 2; ch++) {
@@ -709,18 +814,98 @@ static void display_enum_list(snd_mixer_elem_t *elem, int y, int x)
   }
 }
 
+static void draw_volume_bar(int x, int y, int elem_index, long vleft, long vright)
+{
+  int i, dc;
+
+  mixer_dc (DC_CBAR_FRAME);
+  if (mixer_type[elem_index] & MIXER_ELEM_MUTE_SWITCH) {
+    mvaddch (y, x + 2, ACS_LTEE);
+    mvaddch (y, x + 5, ACS_RTEE);
+  } else {
+    mvaddch (y, x + 2, ACS_LLCORNER);
+    mvaddch (y, x + 3, ACS_HLINE);
+    mvaddch (y, x + 4, ACS_HLINE);
+    mvaddch (y, x + 5, ACS_LRCORNER);
+  }
+  y--;
+  for (i = 0; i < mixer_cbar_height; i++)
+    {
+      mvaddstr (y - i, x, "         ");
+      mvaddch (y - i, x + 2, ACS_VLINE);
+      mvaddch (y - i, x + 5, ACS_VLINE);
+    }
+  for (i = 0; i < mixer_cbar_height; i++)
+    {
+      if (i + 1 >= 0.8 * mixer_cbar_height)
+	dc = DC_ANY_3;
+      else if (i + 1 >= 0.4 * mixer_cbar_height)
+	dc = DC_ANY_2;
+      else
+	dc = DC_ANY_1;
+      mvaddch (y, x + 3, mixer_dc (vleft > i * 100 / mixer_cbar_height ? dc : DC_CBAR_EMPTY));
+	mvaddch (y, x + 4, mixer_dc (vright > i * 100 / mixer_cbar_height ? dc : DC_CBAR_EMPTY));
+	y--;
+      }
+  
+  mixer_dc (DC_CBAR_FRAME);
+  mvaddstr (y, x, "         ");
+  mvaddch (y, x + 2, ACS_ULCORNER);
+  mvaddch (y, x + 3, ACS_HLINE);
+  mvaddch (y, x + 4, ACS_HLINE);
+  mvaddch (y, x + 5, ACS_URCORNER);
+}
+
+static void draw_playback_switch(int x, int y, int elem_index, int swl, int swr)
+{
+  int dc;
+
+  mixer_dc (DC_CBAR_FRAME);
+  mvaddch (y, x + 2, ACS_LLCORNER);
+  mvaddch (y, x + 3, ACS_HLINE);
+  mvaddch (y, x + 4, ACS_HLINE);
+  mvaddch (y, x + 5, ACS_LRCORNER);
+  mvaddstr (y - 1, x, "         ");
+  mvaddch (y - 1, x + 2, ACS_VLINE);
+  mvaddch (y - 1, x + 5, ACS_VLINE);
+  mvaddstr (y - 2, x, "         ");
+  mvaddch (y - 2, x + 2, ACS_ULCORNER);
+  mvaddch (y - 2, x + 3, ACS_HLINE);
+  mvaddch (y - 2, x + 4, ACS_HLINE);
+  mvaddch (y - 2, x + 5, ACS_URCORNER);
+  dc = swl ? DC_CBAR_NOMUTE : DC_CBAR_MUTE;
+  mvaddch (y - 1, x + 3, mixer_dc (dc));
+  dc = swr ? DC_CBAR_NOMUTE : DC_CBAR_MUTE;
+  mvaddch (y - 1, x + 4, mixer_dc (dc));
+}
+
+static void draw_capture_switch(int x, int y, int elem_index, int swl, int swr)
+{
+  int i;
+
+  if (swl || swr) {
+    mixer_dc (DC_CBAR_CAPTURE);
+    mvaddstr (y, x + 1, "CAPTUR");
+  } else {
+    for (i = 0; i < 6; i++)
+      mvaddch(y, x + i + 1, mixer_dc(DC_CBAR_NOCAPTURE));
+  }
+  mixer_dc (DC_CBAR_CAPTURE);
+  mvaddch (y - 1, x + 1, swl ? 'L' : ' ');
+  mvaddch (y - 1, x + 6, swr ? 'R' : ' ');
+}
+
 static void
 mixer_update_cbar (int elem_index)
 {
-  char string[128], string1[64], *suffix;
-  int dc;
   snd_mixer_elem_t *elem;
   long vleft, vright;
   int type;
   snd_mixer_selem_id_t *sid;
   snd_mixer_selem_channel_id_t chn_left, chn_right;
-  int x, y, i;
+  int x, y;
   int swl, swr;
+  char * extra_info;
 
   /* set new scontrol indices and read info
    */
@@ -758,7 +943,7 @@ mixer_update_cbar (int elem_index)
       vright = vleft;
     }
   }
-  
+
   if (type == MIXER_ELEM_CAPTURE && snd_mixer_selem_has_capture_volume(elem)) {
     long vmin, vmax;
     snd_mixer_selem_get_capture_volume_range(elem, &vmin, &vmax);
@@ -775,33 +960,74 @@ mixer_update_cbar (int elem_index)
   /* update the focused full bar name
    */
   if (elem_index == mixer_focus_elem) {
-    int xlen = mixer_max_x - 8;
-    if (xlen > 63)
-      xlen = 63;
-    mixer_dc (DC_PROMPT);
-    mvaddstr (3, 2, "Item: ");
-    mixer_dc (DC_TEXT);
-    string1[8] = 0;
-    for (i = 0; i < 63; i++)
-      string1[i] = ' ';
-    string1[xlen] = '\0';
-    strcpy(string, snd_mixer_selem_id_get_name(sid));
-    if (mixer_type[elem_index] & MIXER_ELEM_CAPTURE_SUFFIX)
-      strcat(string, " Capture");
-    if (snd_mixer_selem_id_get_index(sid) > 0)
-      sprintf(string + strlen(string), " %i", snd_mixer_selem_id_get_index(sid));
-    if ((mixer_type[elem_index] & MIXER_ELEM_MUTE_SWITCH)
-	&& snd_mixer_selem_has_playback_switch(elem)) {
+    char tmp[50];
+    /* control muted? */
+    swl = swr = 1;
+    extra_info = "";
+    if (mixer_type[elem_index] & MIXER_ELEM_MUTE_SWITCH) {
       snd_mixer_selem_get_playback_switch(elem, chn_left, &swl);
-      swr = 0;
+      swr = swl;
       if (chn_right != SND_MIXER_SCHN_UNKNOWN)
 	snd_mixer_selem_get_playback_switch(elem, chn_right, &swr);
-      if (! swl && ! swr)
-	sprintf(string + strlen(string), " [Off]");
+      extra_info = !swl && !swr ? " [Off]" : "";
     }
-    string[xlen] = '\0';
-    strncpy(string1, string, strlen(string));
-    addstr(string1);
+    if (type == MIXER_ELEM_ENUM) {
+      /* FIXME: should show the item names of secondary and later channels... */
+      unsigned int eidx, length;
+      tmp[0]=' ';
+      tmp[1]='[';
+      if (! snd_mixer_selem_get_enum_item(elem, 0, &eidx) &&
+	  ! snd_mixer_selem_get_enum_item_name(elem, eidx, sizeof(tmp) - 3, tmp+2)) {
+	tmp[sizeof(tmp)-2] = 0;
+	length=strlen(tmp);
+	tmp[length]=']';
+	tmp[length+1]=0;
+	extra_info = tmp;
+      }
+    }
+    if (type != MIXER_ELEM_CAPTURE && snd_mixer_selem_has_playback_volume(elem)) {
+	long vdbleft, vdbright;
+	unsigned int length;
+	if (!snd_mixer_selem_get_playback_dB(elem, chn_left, &vdbleft)) {
+		if ((chn_right != SND_MIXER_SCHN_UNKNOWN) &&
+		     (!snd_mixer_selem_get_playback_dB(elem, chn_right, &vdbright))) {
+			float dbvol1, dbvol2;
+			dbvol1=(float)vdbleft/100;
+			dbvol2=(float)vdbright/100;
+    			snprintf(tmp, 48, " [dB gain=%3.2f, %3.2f]",dbvol1, dbvol2);
+		} else {
+			float dbvol1;
+			dbvol1=(float)vdbleft/100;
+    			snprintf(tmp, 48, " [dB gain=%3.2f]",dbvol1);
+		}
+		tmp[sizeof(tmp)-2] = 0;
+		length=strlen(tmp);
+		tmp[length+1]=0;
+		extra_info = tmp;
+	}
+    }
+    if (type == MIXER_ELEM_CAPTURE && snd_mixer_selem_has_capture_volume(elem)) {
+	long vdbleft, vdbright;
+	unsigned int length;
+	if (!snd_mixer_selem_get_capture_dB(elem, chn_left, &vdbleft)) {
+		if ((chn_right != SND_MIXER_SCHN_UNKNOWN) &&
+		     (!snd_mixer_selem_get_capture_dB(elem, chn_right, &vdbright))) {
+			float dbvol1, dbvol2;
+			dbvol1=(float)vdbleft/100;
+			dbvol2=(float)vdbright/100;
+    			snprintf(tmp, 48, " [dB gain=%3.2f, %3.2f]",dbvol1, dbvol2);
+		} else {
+			float dbvol1;
+			dbvol1=(float)vdbleft/100;
+    			snprintf(tmp, 48, " [dB gain=%3.2f]",dbvol1);
+		}
+		tmp[sizeof(tmp)-2] = 0;
+		length=strlen(tmp);
+		tmp[length+1]=0;
+		extra_info = tmp;
+	}
+    }
+    display_item_info(elem_index, sid, extra_info);
   }
 
   /* get channel bar position
@@ -811,138 +1037,84 @@ mixer_update_cbar (int elem_index)
 
   /* channel bar name
    */
-  mixer_dc (elem_index == mixer_focus_elem ? DC_CBAR_FOCUS_LABEL : DC_CBAR_LABEL);
-  if (mixer_type[elem_index] & MIXER_ELEM_CAPTURE_SUFFIX)
-    suffix = " Capture";
-  else
-    suffix = "";
-  if (snd_mixer_selem_id_get_index(sid) > 0)
-    sprintf(string1, "%s%s %d", snd_mixer_selem_id_get_name(sid), suffix, snd_mixer_selem_id_get_index(sid));
-  else
-    sprintf(string1, "%s%s", snd_mixer_selem_id_get_name(sid), suffix);
-  string1[8] = 0;
-  for (i = 0; i < 8; i++)
-    {
-      string[i] = ' ';
-    }
-  sprintf (string + (8 - strlen (string1)) / 2, "%s          ", string1);
-  string[8] = 0;
-  mvaddstr (y, x, string);
+  display_item_name(x, y, elem_index, sid);
   y--;
   
+  /* enum list? */
+  if (type == MIXER_ELEM_ENUM) {
+    display_enum_list(elem, y, x);
+    return; /* no more to display */
+  } 
+
   /* current channel values
    */
   mixer_dc (DC_BACK);
   mvaddstr (y, x, "         ");
   if (mixer_type[elem_index] & MIXER_ELEM_HAS_VOLUME) {
+    char string[4];
     mixer_dc (DC_TEXT);
     if (chn_right == SND_MIXER_SCHN_UNKNOWN) {
       /* mono */
-      sprintf (string, "%ld", vleft);
+      snprintf (string, sizeof(string), "%ld", vleft);
       mvaddstr (y, x + 4 - strlen (string) / 2, string);
     } else {
       /* stereo */
-      sprintf (string, "%ld", vleft);
+      snprintf (string, sizeof(string), "%ld", vleft);
       mvaddstr (y, x + 3 - strlen (string), string);
       mixer_dc (DC_CBAR_FRAME);
       mvaddch (y, x + 3, '<');
       mvaddch (y, x + 4, '>');
       mixer_dc (DC_TEXT);
-      sprintf (string, "%ld", vright);
+      snprintf (string, sizeof(string), "%ld", vright);
       mvaddstr (y, x + 5, string);
     }
   }
   y--;
   
-  if (type == MIXER_ELEM_ENUM) {
-    display_enum_list(elem, y, x);
-    return;
-  } 
-
-  /* left/right bar
-   */
-  mixer_dc (DC_CBAR_FRAME);
-  mvaddstr (y, x, "         ");
-  mvaddch (y, x + 2, ACS_LLCORNER);
-  mvaddch (y, x + 3, ACS_HLINE);
-  mvaddch (y, x + 4, ACS_HLINE);
-  mvaddch (y, x + 5, ACS_LRCORNER);
-  y--;
-  for (i = 0; i < mixer_cbar_height; i++)
-    {
-      mvaddstr (y - i, x, "         ");
-      mvaddch (y - i, x + 2, ACS_VLINE);
-      mvaddch (y - i, x + 5, ACS_VLINE);
-    }
-  string[2] = 0;
-  for (i = 0; i < mixer_cbar_height; i++)
-    {
-      if (i + 1 >= 0.8 * mixer_cbar_height)
-	dc = DC_ANY_3;
-      else if (i + 1 >= 0.4 * mixer_cbar_height)
-	dc = DC_ANY_2;
-      else
-	dc = DC_ANY_1;
-      mvaddch (y, x + 3, mixer_dc (vleft > i * 100 / mixer_cbar_height ? dc : DC_CBAR_EMPTY));
-      mvaddch (y, x + 4, mixer_dc (vright > i * 100 / mixer_cbar_height ? dc : DC_CBAR_EMPTY));
-      y--;
-    }
-  
-  /* muted?
-   */
-  mixer_dc (DC_BACK);
-  mvaddstr (y, x, "         ");
-  if ((mixer_type[elem_index] & MIXER_ELEM_MUTE_SWITCH)
-      && snd_mixer_selem_has_playback_switch(elem)) {
-    mixer_dc (DC_CBAR_FRAME);
-    mvaddch (y, x + 2, ACS_ULCORNER);
-    snd_mixer_selem_get_playback_switch(elem, chn_left, &swl);
-    dc = swl ? DC_CBAR_NOMUTE : DC_CBAR_MUTE;
-    mvaddch (y, x + 3, mixer_dc (dc));
-    if (chn_right != SND_MIXER_SCHN_UNKNOWN) {
-      snd_mixer_selem_get_playback_switch(elem, chn_right, &swr);
-      dc = swr ? DC_CBAR_NOMUTE : DC_CBAR_MUTE;
-    }
-    mvaddch (y, x + 4, mixer_dc (dc));
-    mixer_dc (DC_CBAR_FRAME);
-    mvaddch (y, x + 5, ACS_URCORNER);
-  } else {
-    mixer_dc (DC_CBAR_FRAME);
-    mvaddch (y, x + 2, ACS_ULCORNER);
-    mvaddch (y, x + 3, ACS_HLINE);
-    mvaddch (y, x + 4, ACS_HLINE);
-    mvaddch (y, x + 5, ACS_URCORNER);
-  }    
-  y--;
-  
   /* capture input?
    */
-  if ((mixer_type[elem_index] & MIXER_ELEM_CAPTURE_SWITCH) &&
-      snd_mixer_selem_has_capture_switch(elem)) {
-    int has_r_sw = chn_right != SND_MIXER_SCHN_UNKNOWN &&
-      snd_mixer_selem_has_capture_channel(elem, chn_right);
-    snd_mixer_selem_get_capture_switch(elem, chn_left, &swl);
-    if (has_r_sw)
-      snd_mixer_selem_get_capture_switch(elem, chn_right, &swr);
-    if (swl || (has_r_sw && swr)) {
-      mixer_dc (DC_CBAR_CAPTURE);
-      mvaddstr (y, x + 1, "CAPTUR");
-      if (swl) {
-	mvaddstr (y + 1, x + 1, "L");
-	if (! has_r_sw)
-	  mvaddstr (y + 1, x + 6, "R");
-      }
-      if (has_r_sw && swr)
-	mvaddstr (y + 1, x + 6, "R");
+  if (mixer_view == VIEW_CAPTURE || mixer_view == VIEW_CHANNELS) {
+    if ((mixer_type[elem_index] & MIXER_ELEM_CAPTURE_SWITCH) &&
+	snd_mixer_selem_has_capture_switch(elem)) {
+      int has_r_sw = chn_right != SND_MIXER_SCHN_UNKNOWN &&
+	snd_mixer_selem_has_capture_channel(elem, chn_right);
+      snd_mixer_selem_get_capture_switch(elem, chn_left, &swl);
+      if (has_r_sw)
+	snd_mixer_selem_get_capture_switch(elem, chn_right, &swr);
+      else
+	swr = swl;
+      draw_capture_switch(x, y, elem_index, swl, swr);
+    } else
+      draw_blank(x, y, 2);
+    y--;
+  }
+
+  /* mute switch */
+  if (mixer_view == VIEW_PLAYBACK || mixer_view == VIEW_CHANNELS) {
+    if (mixer_type[elem_index] & MIXER_ELEM_MUTE_SWITCH) {
+      snd_mixer_selem_get_playback_switch(elem, chn_left, &swl);
+      if (chn_right != SND_MIXER_SCHN_UNKNOWN)
+	snd_mixer_selem_get_playback_switch(elem, chn_right, &swr);
+      else
+	swr = swl;
+      draw_playback_switch(x, y, elem_index, swl, swr);
     } else {
-      for (i = 0; i < 6; i++)
-	mvaddch (y, x + 1 + i, mixer_dc (DC_CBAR_NOCAPTURE));
+      mixer_dc (DC_CBAR_FRAME);
+      mvaddstr (y, x + 2, "    ");
+      draw_blank(x, y - 1, 2);
     }
-  } else {
-    mixer_dc (DC_BACK);
-    mvaddstr (y, x, "         ");
+    y -= 2;
+  }
+
+  /* left/right volume bar
+   */
+  if (mixer_type[elem_index] & MIXER_ELEM_HAS_VOLUME)
+    draw_volume_bar(x, y, elem_index, vleft, vright);
+  else {
+    if (mixer_view == VIEW_CAPTURE)
+      mvaddstr (y, x + 2, "    ");
+    draw_blank(x, y - 1, mixer_cbar_height + 1);
   }
-  y--;
 }
 
 static void
@@ -952,7 +1124,7 @@ mixer_update_cbars (void)
   static int o_y = 0;
   int i, x, y;
   
-  
+  display_view_info();
   if (!mixer_cbar_get_pos (mixer_focus_elem, &x, &y))
     {
       if (mixer_focus_elem < mixer_first_vis_elem)
@@ -961,15 +1133,15 @@ mixer_update_cbars (void)
 	mixer_first_vis_elem = mixer_focus_elem - mixer_n_vis_elems + 1;
       mixer_cbar_get_pos (mixer_focus_elem, &x, &y);
     }
-  if (mixer_first_vis_elem + mixer_n_vis_elems >= mixer_n_elems) {
-     mixer_first_vis_elem = mixer_n_elems - mixer_n_vis_elems;
+  if (mixer_first_vis_elem + mixer_n_vis_elems >= mixer_n_view_elems) {
+     mixer_first_vis_elem = mixer_n_view_elems - mixer_n_vis_elems;
      if (mixer_first_vis_elem < 0)
        mixer_first_vis_elem = 0;
      mixer_cbar_get_pos (mixer_focus_elem, &x, &y);
   }
   mixer_write_cbar(mixer_focus_elem);
   for (i = 0; i < mixer_n_vis_elems; i++) {
-    if (i + mixer_first_vis_elem >= mixer_n_elems)
+    if (i + mixer_first_vis_elem >= mixer_n_view_elems)
       continue;
     mixer_update_cbar (i + mixer_first_vis_elem);
   }
@@ -997,8 +1169,6 @@ mixer_draw_frame (void)
   int i;
   int max_len;
   
-  mixer_dc (DC_FRAME);
-  
   /* card name
    */
   mixer_dc (DC_PROMPT);
@@ -1023,7 +1193,7 @@ mixer_draw_frame (void)
 
   /* lines
    */
-  mixer_dc (DC_PROMPT);
+  mixer_dc (DC_FRAME);
   for (i = 1; i < mixer_max_y - 1; i++)
     {
       mvaddch (i, 0, ACS_VLINE);
@@ -1037,7 +1207,6 @@ mixer_draw_frame (void)
   
   /* corners
    */
-  mixer_dc (DC_PROMPT);
   mvaddch (0, 0, ACS_ULCORNER);
   mvaddch (0, mixer_max_x - 1, ACS_URCORNER);
   mvaddch (mixer_max_y - 1, 0, ACS_LLCORNER);
@@ -1050,6 +1219,30 @@ mixer_draw_frame (void)
       mvaddch (mixer_max_y - 1, mixer_max_x - 2, ACS_LRCORNER);
     }
 
+  /* left/right scroll indicators */
+  switch (mixer_view) {
+  case VIEW_PLAYBACK:
+  case VIEW_CAPTURE:
+  case VIEW_CHANNELS:
+    if (mixer_cbar_height > 0) {
+      int ind_hgt = (mixer_cbar_height + 1) / 2;
+      int ind_ofs = mixer_max_y / 2 - ind_hgt/2;
+      /* left scroll possible? */
+      if (mixer_first_vis_elem > 0) {
+	for (i = 0; i < ind_hgt; i++)
+	  mvaddch (i + ind_ofs, 0, '<');
+      }
+      /* right scroll possible? */
+      if (mixer_first_vis_elem + mixer_n_vis_elems < mixer_n_view_elems) {
+	for (i = 0; i < ind_hgt; i++)
+	  mvaddch (i + ind_ofs, mixer_max_x - 1, '>');
+      }
+    }
+    break;
+  default:
+    break;
+  }
+
   /* program title
    */
   sprintf (string, "%s v%s (Press Escape to quit)", PRGNAME_UPPER, VERSION);
@@ -1425,9 +1618,9 @@ mixer_init (void)
    */
   if ((err = snd_mixer_open (&mixer_handle, 0)) < 0)
     mixer_abort (ERR_FCN, "snd_mixer_open", err);
-  if ((err = snd_mixer_attach (mixer_handle, card_id)) < 0)
+  if (mixer_level == 0 && (err = snd_mixer_attach (mixer_handle, card_id)) < 0)
     mixer_abort (ERR_FCN, "snd_mixer_attach", err);
-  if ((err = snd_mixer_selem_register (mixer_handle, NULL, NULL)) < 0)
+  if ((err = snd_mixer_selem_register (mixer_handle, mixer_level > 0 ? &mixer_options : NULL, NULL)) < 0)
     mixer_abort (ERR_FCN, "snd_mixer_selem_register", err);
   snd_mixer_set_callback (mixer_handle, mixer_event);
   if ((err = snd_mixer_load (mixer_handle)) < 0)
@@ -1439,6 +1632,35 @@ mixer_init (void)
   strcpy(mixer_device_name, snd_ctl_card_info_get_mixername(hw_info));
 }
 
+/* init mixer screen
+ */
+static void
+recalc_screen_size (void)
+{
+  getmaxyx (mixer_window, mixer_max_y, mixer_max_x);
+  if (mixer_minimize)
+    {
+      mixer_max_x = MIXER_MIN_X;
+      mixer_max_y = MIXER_MIN_Y;
+    }
+  mixer_ofs_x = 2 /* extra begin padding: */ + 1;
+
+  /* required allocations */
+  mixer_n_vis_elems = (mixer_max_x - mixer_ofs_x * 2 + 1) / 9;
+  mixer_n_vis_elems = CLAMP (mixer_n_vis_elems, 1, mixer_n_view_elems);
+  mixer_extra_space = mixer_max_x - mixer_ofs_x * 2 + 1 - mixer_n_vis_elems * 9;
+  mixer_extra_space = MAX (0, mixer_extra_space / (mixer_n_vis_elems + 1));
+  mixer_text_y = MIXER_TEXT_Y;
+  if (mixer_view == VIEW_PLAYBACK || mixer_view == VIEW_CHANNELS)
+    mixer_text_y += 2; /* row for mute switch */
+  if (mixer_view == VIEW_CAPTURE || mixer_view == VIEW_CHANNELS)
+    mixer_text_y++; /* row for capture switch */
+  if (mixer_text_y + MIXER_CBAR_STD_HGT < mixer_max_y)
+    mixer_cbar_height = MIXER_CBAR_STD_HGT + MAX (1, mixer_max_y - mixer_text_y - MIXER_CBAR_STD_HGT + 1) / 2;
+  else
+    mixer_cbar_height = MAX (1, mixer_max_y - mixer_text_y);
+}
+
 static void
 mixer_reinit (void)
 {
@@ -1504,12 +1726,12 @@ __again:
 
   if (mixer_type)
     free(mixer_type);
-  mixer_type = (int *)malloc(sizeof(int) * mixer_n_elems);
+  mixer_type = (int *)calloc(mixer_n_elems, sizeof(int));
   if (mixer_type == NULL)
     mixer_abort(ERR_FCN, "malloc", 0);
   if (mixer_grpidx)
     free(mixer_grpidx);
-  mixer_grpidx = (int *)malloc(sizeof(int) * mixer_n_elems);
+  mixer_grpidx = (int *)calloc(mixer_n_elems, sizeof(int));
   if (mixer_grpidx == NULL)
     mixer_abort(ERR_FCN, "malloc", 0);
   elem_index = 0;
@@ -1521,53 +1743,64 @@ __again:
     elem = snd_mixer_find_selem(mixer_handle, sid);
     if (elem == NULL)
       CHECK_ABORT (ERR_FCN, "snd_mixer_find_selem()", -EINVAL);
-    for (i = 0; i < MIXER_ELEM_CAPTURE; i++) {
-      int ok;
-      for (j = ok = 0; j < 2; j++) {
-	if (mixer_changed_state)
-	  goto __again;
-	if (snd_mixer_selem_has_playback_channel(elem, mixer_elem_chn[i][j]))
-	  ok++;
+    if ( (mixer_view == VIEW_PLAYBACK) || (mixer_view == VIEW_CHANNELS) ) {
+      for (i = 0; i < MIXER_ELEM_CAPTURE; i++) {
+        int ok;
+        for (j = ok = 0; j < 2; j++) {
+	  if (mixer_changed_state)
+	    goto __again;
+ 	  if (snd_mixer_selem_has_playback_channel(elem, mixer_elem_chn[i][j]))
+	    ok++;
+        }
+        if (ok) {
+	  sid = (snd_mixer_selem_id_t *)(((char *)mixer_sid) + snd_mixer_selem_id_sizeof() * idx);
+	  mixer_grpidx[elem_index] = idx;
+	  if (snd_mixer_selem_is_enumerated(elem)) {
+	    mixer_type[elem_index] = MIXER_ELEM_ENUM;
+	  } else {
+	    mixer_type[elem_index] = i;
+	    if (i == 0 && snd_mixer_selem_has_playback_switch(elem))
+	      mixer_type[elem_index] |= MIXER_ELEM_MUTE_SWITCH;
+	    if (snd_mixer_selem_has_playback_volume(elem))
+	      mixer_type[elem_index] |= MIXER_ELEM_HAS_VOLUME;
+	  }
+	  if (mixer_view == VIEW_CHANNELS) {
+	    if (nelems_added == 0 &&
+		! snd_mixer_selem_has_capture_volume(elem) &&
+		snd_mixer_selem_has_capture_switch(elem))
+	      mixer_type[elem_index] |= MIXER_ELEM_CAPTURE_SWITCH;
+	  }
+	  elem_index++;
+	  nelems_added++;
+	  if (elem_index >= mixer_n_elems)
+	    break;
+        }
       }
-      if (ok) {
-	sid = (snd_mixer_selem_id_t *)(((char *)mixer_sid) + snd_mixer_selem_id_sizeof() * idx);
-	mixer_grpidx[elem_index] = idx;
-	if (snd_mixer_selem_is_enumerated(elem)) {
-	  mixer_type[elem_index] = MIXER_ELEM_ENUM;
-	} else {
-	  mixer_type[elem_index] = i;
-	  if (i == 0 && snd_mixer_selem_has_playback_switch(elem))
-	    mixer_type[elem_index] |= MIXER_ELEM_MUTE_SWITCH;
-	  if (snd_mixer_selem_has_playback_volume(elem))
-	    mixer_type[elem_index] |= MIXER_ELEM_HAS_VOLUME;
-	  if (i == 0 && snd_mixer_selem_has_capture_switch(elem))
-	    mixer_type[elem_index] |= MIXER_ELEM_CAPTURE_SWITCH;
-	}
-	elem_index++;
-	nelems_added++;
-	if (elem_index >= mixer_n_elems)
+    }
+
+    if ( (mixer_view == VIEW_CAPTURE) || (mixer_view == VIEW_CHANNELS) ) {
+      if (snd_mixer_selem_has_capture_volume(elem) ||
+	  (nelems_added == 0 && snd_mixer_selem_has_capture_switch(elem))) {
+        mixer_grpidx[elem_index] = idx;
+        mixer_type[elem_index] = MIXER_ELEM_CAPTURE;
+        if (nelems_added == 0 && snd_mixer_selem_has_capture_switch(elem))
+	  mixer_type[elem_index] |= MIXER_ELEM_CAPTURE_SWITCH;
+        if (nelems_added)
+	  mixer_type[elem_index] |= MIXER_ELEM_CAPTURE_SUFFIX;
+        if (snd_mixer_selem_has_capture_volume(elem))
+	  mixer_type[elem_index] |= MIXER_ELEM_HAS_VOLUME;
+        elem_index++;
+        if (elem_index >= mixer_n_elems)
 	  break;
       }
     }
-    if (snd_mixer_selem_has_capture_volume(elem) ||
-	(nelems_added == 0 && snd_mixer_selem_has_capture_switch(elem))) {
-      mixer_grpidx[elem_index] = idx;
-      mixer_type[elem_index] = MIXER_ELEM_CAPTURE;
-      if (nelems_added == 0 && snd_mixer_selem_has_capture_switch(elem))
-	mixer_type[elem_index] |= MIXER_ELEM_CAPTURE_SWITCH;
-      if (nelems_added)
-	mixer_type[elem_index] |= MIXER_ELEM_CAPTURE_SUFFIX;
-      if (snd_mixer_selem_has_capture_volume(elem))
-	mixer_type[elem_index] |= MIXER_ELEM_HAS_VOLUME;
-      elem_index++;
-      if (elem_index >= mixer_n_elems)
-	break;
-    }
   }
 
+  mixer_n_view_elems = elem_index;
+  recalc_screen_size();
   mixer_focus_elem = 0;
   if (focus_type >= 0) {
-    for (elem_index = 0; elem_index < mixer_n_elems; elem_index++) {
+    for (elem_index = 0; elem_index < mixer_n_view_elems; elem_index++) {
       sid = (snd_mixer_selem_id_t *)(((char *)mixer_sid) + snd_mixer_selem_id_sizeof() * mixer_grpidx[elem_index]);
       if (!strcmp(snd_mixer_selem_id_get_name(focus_gid),
                   snd_mixer_selem_id_get_name(sid)) &&
@@ -1607,25 +1840,7 @@ mixer_init_window (void)
   keypad (mixer_window, TRUE);
   GETCH_BLOCK (1);
 
-  /* init mixer screen
-   */
-  getmaxyx (mixer_window, mixer_max_y, mixer_max_x);
-  if (mixer_minimize)
-    {
-      mixer_max_x = MIXER_MIN_X;
-      mixer_max_y = MIXER_MIN_Y;
-    }
-  mixer_ofs_x = 2 /* extra begin padding: */ + 1;
-
-  /* required allocations */
-  mixer_n_vis_elems = (mixer_max_x - mixer_ofs_x * 2 + 1) / 9;
-  mixer_n_vis_elems = CLAMP (mixer_n_vis_elems, 1, mixer_n_elems);
-  mixer_extra_space = mixer_max_x - mixer_ofs_x * 2 + 1 - mixer_n_vis_elems * 9;
-  mixer_extra_space = MAX (0, mixer_extra_space / (mixer_n_vis_elems + 1));
-  if (MIXER_TEXT_Y + 10 < mixer_max_y)
-    mixer_cbar_height = 10 + MAX (0, mixer_max_y - MIXER_TEXT_Y - 10 ) / 2;
-  else
-    mixer_cbar_height = MAX (1, mixer_max_y - MIXER_TEXT_Y);
+  recalc_screen_size();
 
   mixer_clear (TRUE);
 }
@@ -1764,18 +1979,22 @@ mixer_iteration (void)
       /* ignore */
       break;
     case 27:	/* Escape */
+    case KEY_F (10):
       finished = 1;
       key = 0;
       break;
     case 13:	/* Return */
     case 10:	/* NewLine */
-      if (mixer_view == VIEW_CHANNELS)
-	mixer_clear (FALSE);
-      mixer_view = VIEW_CHANNELS;
+      if (mixer_view != mixer_view_saved) {
+        mixer_view = mixer_view_saved;
+        mixer_changed_state=1;
+        mixer_reinit ();
+      }
       key = 0;
       break;
     case 'h':
     case 'H':
+    case '?':
     case KEY_F (1):
       mixer_view = VIEW_HELP;
       key = 0;
@@ -1785,6 +2004,45 @@ mixer_iteration (void)
       mixer_view = VIEW_PROCINFO;
       key = 0;
       break;
+    case KEY_F (3):
+      if (mixer_view == VIEW_PLAYBACK) {
+	mixer_clear (FALSE);
+      } else {
+        mixer_view = mixer_view_saved = VIEW_PLAYBACK;
+        mixer_changed_state=1;
+        mixer_reinit ();
+      } 
+      key = 0;
+      break;
+    case KEY_F (4):
+      if (mixer_view == VIEW_CAPTURE) {
+	mixer_clear (FALSE);
+      } else {
+        mixer_view = mixer_view_saved = VIEW_CAPTURE;
+        mixer_changed_state=1;
+        mixer_reinit ();
+      } 
+      key = 0;
+      break;
+    case KEY_F (5):
+      if (mixer_view == VIEW_CHANNELS) {
+	mixer_clear (FALSE);
+      } else {
+        mixer_view = mixer_view_saved = VIEW_CHANNELS;
+        mixer_changed_state=1;
+        mixer_reinit ();
+      } 
+      key = 0;
+      break;
+    case 9: /* Tab */
+      if (mixer_view >= VIEW_CHANNELS && mixer_view <= VIEW_CAPTURE) {
+	mixer_view = (mixer_view + 1) % 3 + VIEW_CHANNELS;
+	mixer_view_saved = mixer_view;
+	mixer_changed_state = 1;
+	mixer_reinit ();
+	key = 0;
+      }
+      break;
     case '\014':
     case 'L':
     case 'l':
@@ -1855,7 +2113,10 @@ mixer_iteration (void)
 	break;
       }
   
-  if (key && mixer_view == VIEW_CHANNELS)
+  if (key && 
+    ((mixer_view == VIEW_CHANNELS) ||
+    (mixer_view == VIEW_PLAYBACK) ||
+    (mixer_view == VIEW_CAPTURE)) )
     switch (key)
       {
       case KEY_RIGHT:
@@ -1900,6 +2161,18 @@ mixer_iteration (void)
       case 'X':
         mixer_add_delta(-1);
 	break;
+      case '0':
+      case '1':
+      case '2':
+      case '3':
+      case '4':
+      case '5':
+      case '6':
+      case '7':
+      case '8':
+      case '9':
+	mixer_volume_absolute = 10 * (key - '0');
+	break;
       case 'q':
 	mixer_volume_delta[MIXER_CHN_LEFT] = 1;
       case 'Q':
@@ -1955,7 +2228,10 @@ mixer_iteration (void)
   if (old_view != mixer_view)
     mixer_clear (FALSE);
   
-  mixer_focus_elem = CLAMP (mixer_focus_elem, 0, mixer_n_elems - 1);
+  if (! mixer_n_view_elems)
+    mixer_focus_elem = 0;
+  else
+    mixer_focus_elem = CLAMP (mixer_focus_elem, 0, mixer_n_view_elems - 1);
   
   return finished;
 }
@@ -1972,7 +2248,7 @@ static void
 mixer_signal_handler (int signal)
 {
   if (signal != SIGSEGV)
-    mixer_abort (ERR_SIGNAL, sys_siglist[signal], 0);
+    mixer_abort (ERR_SIGNAL, strsignal(signal), 0);
   else
     {
       fprintf (stderr, "\nSegmentation fault.\n");
@@ -1990,14 +2266,14 @@ main (int    argc,
    */
   do
     {
-      opt = getopt (argc, argv, "c:D:shg");
+      opt = getopt (argc, argv, "c:D:shgV:a:");
       switch (opt)
 	{
 	case '?':
 	case 'h':
-	  fprintf (stderr, "%s v%s\n", PRGNAME_UPPER, VERSION);
-	  fprintf (stderr, "Usage: %s [-h] [-c <card: 0...7 or id>] [-D <mixer device>] [-g] [-s]\n", PRGNAME);
-	  mixer_abort (ERR_NONE, "", 0);
+	  printf ("%s v%s\n", PRGNAME_UPPER, VERSION);
+	  printf ("Usage: %s [-h] [-c <card: 0...7>] [-D <mixer device>] [-g] [-s] [-V <view>] [-a <abst>]\n", PRGNAME);
+	  return 1;
 	case 'c':
 	  {
 	    int i = snd_card_get_index(optarg);
@@ -2018,10 +2294,33 @@ main (int    argc,
 	case 's':
 	  mixer_minimize = 1;
 	  break;
+	case 'V':
+	  if (*optarg == 'p' || *optarg == 'P')
+	    mixer_view = VIEW_PLAYBACK;
+	  else if (*optarg == 'c' || *optarg == 'C')
+	    mixer_view = VIEW_CAPTURE;
+	  else
+	    mixer_view = VIEW_CHANNELS;
+	  break;
+	case 'a':
+	  mixer_level = 1;
+	  memset(&mixer_options, 0, sizeof(mixer_options));
+	  mixer_options.ver = 1;
+	  if (!strcmp(optarg, "none"))
+	    mixer_options.abstract = SND_MIXER_SABSTRACT_NONE;
+	  else if (!strcmp(optarg, "basic"))
+	    mixer_options.abstract = SND_MIXER_SABSTRACT_BASIC;
+	  else {
+	    fprintf(stderr, "Select correct abstraction level (none or basic)...\n");
+	    mixer_abort (ERR_NONE, "", 0);
+	  }
+	  break;
 	}
     }
   while (opt > 0);
   
+  mixer_options.device = card_id;
+  
   /* initialize mixer
    */
   mixer_init ();
@@ -2060,6 +2359,8 @@ main (int    argc,
 	  switch (mixer_view)
 	    {
 	    case VIEW_CHANNELS:
+	    case VIEW_PLAYBACK:
+	    case VIEW_CAPTURE:
 	      mixer_update_cbars ();
 	      break;
 	    case VIEW_HELP:
@@ -2076,4 +2377,4 @@ main (int    argc,
   while (!mixer_iteration ());
   
   mixer_abort (ERR_NONE, "", 0);
-};
+}
